@@ -7,7 +7,6 @@ from typing import (
 import traceback
 from dataclasses import dataclass, replace
 from weakref import WeakKeyDictionary
-from abc import ABC
 from syncraft.ast import ThenKind, Then, Choice, Many, ChoiceKind, shallow_dict
 from syncraft.constraint import Bindable
 
@@ -75,12 +74,16 @@ class Error:
 
 
 @dataclass(frozen=True)        
-class Algebra(ABC, Generic[A, S]):
+class Algebra(Generic[A, S]):
 ######################################################## shared among all subclasses ########################################################
     run_f: Callable[[S, bool], Either[Any, Tuple[A, S]]] 
     name: Hashable
     _cache: ClassVar[WeakKeyDictionary[Any, Dict[Any, object | Either[Any, Tuple[Any, Any]]]]] = WeakKeyDictionary()
 
+    @classmethod
+    def state(cls, *args:Any, **kwargs:Any)->Optional[S]: 
+        return None
+        
     def named(self, name: Hashable) -> 'Algebra[A, S]':
         return replace(self, name=name)
 
@@ -90,8 +93,38 @@ class Algebra(ABC, Generic[A, S]):
     def __call__(self, input: S, use_cache: bool) -> Either[Any, Tuple[A, S]]:
         return self.run(input, use_cache=use_cache)
 
-    
     def run(self, input: S, use_cache: bool) -> Either[Any, Tuple[A, S]]:
+        """Execute this algebra on the given state with optional memoization.
+
+        This is the core evaluation entry point used by all combinators. It
+        supports per-"parser" memoization and protects against infinite
+        recursion by detecting left-recursive re-entrance.
+
+        Args:
+            input: The initial state to run against. Must be hashable as it's
+                used as a cache key.
+            use_cache: When True, memoize results for ``(self.run_f, input)``
+                so repeated calls short-circuit. When False, the cache entry is
+                cleared after the run to effectively disable the cache. 
+
+        Returns:
+            Either[Error, Tuple[A, S]]: On success, ``Right((value, next_state))``.
+            On failure, ``Left(Error)``. Errors produced downstream are
+            automatically enriched with ``this=self`` and ``state=input`` to
+            preserve context. If an exception escapes the user code, it's
+            captured and returned as ``Left(Error)`` with a traceback in
+            ``stack``.
+
+        Notes:
+            - Left recursion: if a re-entrant call is observed on the same
+              ``input`` while an evaluation is in progress, a ``Left(Error)``
+              is returned indicating left-recursion was detected.
+            - Memoization scope: results are cached per ``run_f`` (the concrete
+              compiled function of this algebra) and keyed by the input state.
+            - Commitment: downstream combinators (e.g. ``cut``) may set the
+              ``committed`` flag in ``Error``; ``run`` preserves that flag but
+              does not set it itself.
+        """
         cache = self._cache[self.run_f]
         assert cache is not None, "Cache should be initialized in __post_init__"
         if input in cache:
@@ -133,12 +166,31 @@ class Algebra(ABC, Generic[A, S]):
         
     @classmethod
     def lazy(cls, thunk: Callable[[], Algebra[A, S]]) -> Algebra[A, S]:
+        """Lazily construct an algebra at run time.
+
+        Useful for recursive definitions. The thunk is evaluated when this
+        algebra runs, and the resulting algebra is executed.
+
+        Args:
+            thunk: Zero-argument function returning the underlying algebra.
+
+        Returns:
+            An algebra that defers to the thunk-provided algebra.
+        """
         def lazy_run(input: S, use_cache:bool) -> Either[Any, Tuple[A, S]]:
             return thunk().run(input, use_cache)
         return cls(lazy_run, name=cls.__name__ + '.lazy')
     
     @classmethod
     def fail(cls, error: Any) -> Algebra[Any, S]:
+        """Return an algebra that always fails with ``error``.
+
+        Args:
+            error: The error payload to wrap in ``Left``.
+
+        Returns:
+            An algebra producing ``Left(Error(...))`` without consuming input.
+        """
         def fail_run(input: S, use_cache:bool) -> Either[Any, Tuple[Any, S]]:
             return Left(Error(
                 error=error,
@@ -149,12 +201,35 @@ class Algebra(ABC, Generic[A, S]):
     
     @classmethod
     def success(cls, value: Any) -> Algebra[Any, S]:
+        """Return an algebra that always succeeds with ``value``.
+
+        The input state is passed through unchanged.
+
+        Args:
+            value: The constant value to return.
+
+        Returns:
+            ``Right((value, input))`` for any input state.
+        """
         def success_run(input: S, use_cache:bool) -> Either[Any, Tuple[Any, S]]:
             return Right((value, input))
         return cls(success_run, name=cls.__name__ + '.success')
     
     @classmethod
     def factory(cls, name: str, *args: Any, **kwargs: Any) -> Algebra[A, S]:
+        """Call a named class method to construct an algebra.
+
+        Args:
+            name: Name of a classmethod/staticmethod on this class.
+            *args: Positional args passed to the method.
+            **kwargs: Keyword args passed to the method.
+
+        Returns:
+            The algebra returned by the method.
+
+        Raises:
+            ValueError: If the method is missing or not callable.
+        """
         method = getattr(cls, name, None)
         if method is None or not callable(method):
             raise ValueError(f"Method {name} is not defined in {cls.__name__}")
@@ -163,6 +238,14 @@ class Algebra(ABC, Generic[A, S]):
 
 
     def cut(self) -> Algebra[A, S]:
+        """Commit this branch by marking failures as committed.
+
+        Converts downstream errors into committed errors (``committed=True``),
+        which prevents alternatives from being tried in ``or_else``.
+
+        Returns:
+            An algebra that commits errors produced by this one.
+        """
         def commit_error(e: Any) -> Error:
             match e:
                 case Error():
@@ -185,6 +268,15 @@ class Algebra(ABC, Generic[A, S]):
                     ], 
                     Either[Any, Tuple[B, S]]], 
                     ctx: Optional[Any] = None) -> Algebra[A | B, S]:
+        """Run a handler only when this algebra fails.
+
+        Args:
+            func: Callback ``(alg, input, left, ctx) -> Either`` executed on failure.
+            ctx: Optional context object passed to the callback.
+
+        Returns:
+            An algebra that intercepts failures and can recover or transform them.
+        """
         assert callable(func), "func must be callable"
         def fail_run(input: S, use_cache:bool) -> Either[Any, Tuple[A | B, S]]:
             result = self.run(input, use_cache)
@@ -203,6 +295,15 @@ class Algebra(ABC, Generic[A, S]):
                         ], 
                         Either[Any, Tuple[B, S]]], 
                         ctx: Optional[Any] = None) -> Algebra[A | B, S]:
+        """Run a handler only when this algebra succeeds.
+
+        Args:
+            func: Callback ``(alg, input, right, ctx) -> Either`` executed on success.
+            ctx: Optional context object passed to the callback.
+
+        Returns:
+            An algebra that can transform or post-process successes.
+        """
         assert callable(func), "func must be callable"
         def success_run(input: S, use_cache:bool) -> Either[Any, Tuple[A | B, S]]:
             result = self.run(input, use_cache)
@@ -241,24 +342,29 @@ class Algebra(ABC, Generic[A, S]):
         return lazy_self
 
 ######################################################## map on state ###########################################
-    def post_state(self, f: Callable[[S], S]) -> Algebra[A, S]:
-        def post_state_run(input: S, use_cache:bool) -> Either[Any, Tuple[A, S]]:
-            match self.run(input, use_cache):
-                case Right((value, state)):
-                    return Right((value, f(state)))
-                case Left(err):
-                    return Left(err)
-                case x:
-                    raise ValueError(f"Unexpected result from self.run {x}")
-        return self.__class__(post_state_run, name=self.name) 
+    def map_state(self, f: Callable[[S], S]) -> Algebra[A, S]:
+        """Map the input state before running this algebra.
 
-    def pre_state(self, f: Callable[[S], S]) -> Algebra[A, S]:
-        def pre_state_run(state: S, use_cache:bool) -> Either[Any, Tuple[A, S]]:
+        Args:
+            f: ``S -> S`` function applied to the state prior to running.
+
+        Returns:
+            An algebra that runs with ``f(state)``.
+        """
+        def map_state_run(state: S, use_cache:bool) -> Either[Any, Tuple[A, S]]:
             return self.run(f(state), use_cache)
-        return self.__class__(pre_state_run, name=self.name) 
+        return self.__class__(map_state_run, name=self.name) 
 
 
     def map_all(self, f: Callable[[A, S], Tuple[B, S]]) -> Algebra[B, S]:
+        """Map both the produced value and the resulting state on success.
+
+        Args:
+            f: Function mapping ``(value, state)`` to ``(new_value, new_state)``.
+
+        Returns:
+            An algebra producing the transformed value and state.
+        """
         def map_all_run(input: S, use_cache:bool) -> Either[Any, Tuple[B, S]]:
             match self.run(input, use_cache):
                 case Right((value, state)):
@@ -270,23 +376,54 @@ class Algebra(ABC, Generic[A, S]):
                     raise ValueError(f"Unexpected result from self.run {x}")
         return self.__class__(map_all_run, name=self.name) # type: ignore
 ######################################################## fundamental combinators ############################################    
-    def fmap(self, f: Callable[[A], B]) -> Algebra[B, S]:
-        def fmap_run(input: S, use_cache:bool) -> Either[Any, Tuple[B, S]]:
+    def map(self, f: Callable[[A], B]) -> Algebra[B, S]:
+        """Transform the success value, leaving the state unchanged.
+
+        Args:
+            f: Mapper from ``A`` to ``B``.
+
+        Returns:
+            An algebra that yields ``B`` with the same resulting state.
+        """
+        def map_run(input: S, use_cache:bool) -> Either[Any, Tuple[B, S]]:
             parsed = self.run(input, use_cache)
             if isinstance(parsed, Right):
                 return Right((f(parsed.value[0]), parsed.value[1]))            
             else:
                 return cast(Either[Any, Tuple[B, S]], parsed)
-        return self.__class__(fmap_run, name=self.name)  # type: ignore
+        return self.__class__(map_run, name=self.name)  # type: ignore
 
-    
-    def map(self, f: Callable[[A], B]) -> Algebra[B, S]:
-        return self.fmap(f)
-    
+        
     def bimap(self, f: Callable[[A], B], i: Callable[[B], A]) -> Algebra[B, S]:
-        return self.fmap(f).pre_state(lambda s: s.map(i))
+        """Bidirectionally map values with an inverse, updating the state.
+
+        Applies ``f`` to the success value. The state is pre-mapped with the
+        inverse ``i`` via the state's ``map`` method to preserve round-trips.
+
+        Args:
+            f: Forward mapping ``A -> B``.
+            i: Inverse mapping ``B -> A`` applied to the state.
+
+        Returns:
+            An algebra producing ``B`` while keeping value/state alignment.
+        
+        Note:
+            Different subclass of Algebra can override state.map method to change 
+            the behavior of bimap. For example, ParserState.map will return the
+            state unchanged, and GenState.map will apply the inverse map and update 
+            the next AST node for generation.
+        """
+        return self.map(f).map_state(lambda s: s.map(i))
 
     def map_error(self, f: Callable[[Optional[Any]], Any]) -> Algebra[A, S]:
+        """Transform the error payload when this algebra fails.
+
+        Args:
+            f: Function applied to the error payload inside ``Left``.
+
+        Returns:
+            An algebra that preserves successes and maps failures.
+        """
         def map_error_run(input: S, use_cache:bool) -> Either[Any, Tuple[A, S]]:
             parsed = self.run(input, use_cache)
             if isinstance(parsed, Left):
@@ -295,6 +432,17 @@ class Algebra(ABC, Generic[A, S]):
         return self.__class__(map_error_run, name=self.name)  
 
     def flat_map(self, f: Callable[[A], Algebra[B, S]]) -> Algebra[B, S]:
+        """Chain computations where the next algebra depends on the value.
+
+        On success, passes the produced value to ``f`` to obtain the next
+        algebra, then runs it with the resulting state.
+
+        Args:
+            f: Mapper from a value to the next algebra.
+
+        Returns:
+            An algebra yielding the result of the chained computation.
+        """
         def flat_map_run(input: S, use_cache:bool) -> Either[Any, Tuple[B, S]]:
             parsed = self.run(input, use_cache)
             if isinstance(parsed, Right):
@@ -305,6 +453,18 @@ class Algebra(ABC, Generic[A, S]):
 
     
     def or_else(self: Algebra[A, S], other: Algebra[B, S]) -> Algebra[Choice[A, B], S]:
+        """Try this algebra; if it fails uncommitted, try ``other``.
+
+        If the failure is committed (``committed=True``), the alternative is
+        not attempted and the error is propagated.
+
+        Args:
+            other: Fallback algebra to try from the same input state.
+
+        Returns:
+            An algebra producing ``Choice.LEFT`` for this success or
+            ``Choice.RIGHT`` for the other's success.
+        """
         def or_else_run(input: S, use_cache:bool) -> Either[Any, Tuple[Choice[A, B], S]]:
             match self.run(input, use_cache):
                 case Right((value, state)):
@@ -322,27 +482,75 @@ class Algebra(ABC, Generic[A, S]):
         return self.__class__(or_else_run, name=f'{self.name} | {other.name}')  # type: ignore
 
     def then_both(self, other: Algebra[B, S]) -> Algebra[Then[A, B], S]:
+        """Sequence two algebras and keep both values.
+
+        Returns a ``Then(kind=BOTH)`` holding the left and right values.
+
+        Args:
+            other: The algebra to run after this one.
+
+        Returns:
+            An algebra producing ``Then(left, right, kind=BOTH)``.
+        """
         def then_both_f(a: A) -> Algebra[Then[A, B], S]:
             def combine(b: B) -> Then[A, B]:
                 return Then(left=a, right=b, kind=ThenKind.BOTH)
-            return other.fmap(combine)
+            return other.map(combine)
         return self.flat_map(then_both_f).named(f'{self.name} + {other.name}')
 
     def then_left(self, other: Algebra[B, S]) -> Algebra[Then[A, B], S]:
+        """Sequence two algebras, keep the left value in the result.
+
+        Produces ``Then(kind=LEFT)`` with both values attached.
+
+        Args:
+            other: The algebra to run after this one.
+
+        Returns:
+            An algebra producing ``Then(left, right, kind=LEFT)``.
+        """
         def then_left_f(a: A) -> Algebra[Then[A, B], S]:
             def combine(b: B) -> Then[A, B]:
                 return Then(left=a, right=b, kind=ThenKind.LEFT)
-            return other.fmap(combine)
+            return other.map(combine)
         return self.flat_map(then_left_f).named(f'{self.name} // {other.name}')
 
     def then_right(self, other: Algebra[B, S]) -> Algebra[Then[A, B], S]:
+        """Sequence two algebras, keep the right value in the result.
+
+        Produces ``Then(kind=RIGHT)`` with both values attached.
+
+        Args:
+            other: The algebra to run after this one.
+
+        Returns:
+            An algebra producing ``Then(left, right, kind=RIGHT)``.
+        """
         def then_right_f(a: A) -> Algebra[Then[A, B], S]:
             def combine(b: B) -> Then[A, B]:
                 return Then(left=a, right=b, kind=ThenKind.RIGHT)
-            return other.fmap(combine)
+            return other.map(combine)
         return self.flat_map(then_right_f).named(f'{self.name} >> {other.name}')
 
     def many(self, *, at_least: int, at_most: Optional[int]) -> Algebra[Many[A], S]:
+        """Repeat this algebra and collect results into ``Many``.
+
+        Repeats greedily until failure or no progress. Enforces cardinality
+        constraints. If ``at_most`` is ``None``, there is no upper bound.
+
+        Args:
+            at_least: Minimum number of matches required (>= 1).
+            at_most: Optional maximum number of matches.
+
+        Returns:
+            On success, ``Right((Many(values), state))``.
+        Note:
+            at_most, if given, is enforced strictly, more than at_most matches 
+            is treated as an error.
+        Raises:
+            ValueError: If bounds are invalid (e.g., ``at_least<=0`` or
+            ``at_most<at_least``).
+        """
         if at_least <=0 or (at_most is not None and at_most < at_least):
             raise ValueError(f"Invalid arguments for many: at_least={at_least}, at_most={at_most}")
         def many_run(input: S, use_cache:bool) -> Either[Any, Tuple[Many[A], S]]:
