@@ -12,31 +12,33 @@ def test1_simple_then() -> None:
     A, B, C = literal("a"), literal("b"), literal("c")
     syntax = A // B // C
     sql = "a b c"
-    ast = parse(syntax, sql, dialect="sqlite")
+    ast, bound = parse(syntax, sql, dialect="sqlite")
     print("---" * 40)
     print(ast)
-    generated = gen.generate(syntax, ast)
+    generated, bound = gen.generate(syntax, ast)
     print("---" * 40)
     print(generated)
     assert ast == generated
     value, bmap = generated.bimap()
     print(value)
-    assert gen.generate(syntax, bmap(value)) == generated
+    u, v = gen.generate(syntax, bmap(value))
+    assert u == generated
 
 
 def test2_named_results() -> None:
     A, B = literal("a").mark("x").mark('z'), literal("b").mark("y")
     syntax = A // B
     sql = "a b"
-    ast = parse(syntax, sql, dialect="sqlite")
+    ast, bound = parse(syntax, sql, dialect="sqlite")
     print("---" * 40)
     print(ast)
-    generated = gen.generate(syntax, ast)
+    generated, bound = gen.generate(syntax, ast)
     print("---" * 40)
     print(generated)
     assert ast == generated
     value, bmap = generated.bimap()
-    assert gen.generate(syntax, bmap(value)) == generated
+    u,v = gen.generate(syntax, bmap(value))
+    assert u == generated
     
 
 
@@ -44,15 +46,16 @@ def test3_many_literals() -> None:
     A = literal("a")
     syntax = A.many()
     sql = "a a a"
-    ast = parse(syntax, sql, dialect="sqlite")
+    ast, bound = parse(syntax, sql, dialect="sqlite")
     print("---" * 40)
     print(ast)
-    generated = gen.generate(syntax, ast)
+    generated, bound = gen.generate(syntax, ast)
     print("---" * 40)
     print(generated)
     assert ast == generated
     value, bmap = generated.bimap()
-    assert gen.generate(syntax, bmap(value)) == generated
+    u, v = gen.generate(syntax, bmap(value))
+    assert u == generated
 
 
 def test4_mixed_many_named() -> None:
@@ -60,30 +63,32 @@ def test4_mixed_many_named() -> None:
     B = literal("b")
     syntax = (A | B).many()
     sql = "a b a"
-    ast = parse(syntax, sql, dialect="sqlite")
+    ast, bound = parse(syntax, sql, dialect="sqlite")
     print("---" * 40)
     print(ast)
-    generated = gen.generate(syntax, ast)
+    generated, bound = gen.generate(syntax, ast)
     print("---" * 40)
     print(generated)
     assert ast == generated
     value, bmap = generated.bimap()
-    assert gen.generate(syntax, bmap(value)) == generated
+    u, v = gen.generate(syntax, bmap(value))
+    assert u == generated
 
 
 def test5_nested_then_many() -> None:
     IF, THEN, END = literal("if"), literal("then"), literal("end")
     syntax = (IF.many() // THEN.many()).many() // END
     sql = "if if then end"
-    ast = parse(syntax, sql, dialect="sqlite")
+    ast, bound = parse(syntax, sql, dialect="sqlite")
     print("---" * 40)
     print(ast)
-    generated = gen.generate(syntax, ast, restore_pruned=True)
+    generated, bound = gen.generate(syntax, ast, restore_pruned=True)
     print("---" * 40)
     print(generated)
     assert ast == generated
     value, bmap = generated.bimap()
-    assert gen.generate(syntax, bmap(value), restore_pruned=True) == generated
+    u, v = gen.generate(syntax, bmap(value), restore_pruned=True)
+    assert u == generated
 
 
 
@@ -91,12 +96,13 @@ def test_then_flatten():
     A, B, C = literal("a"), literal("b"), literal("c")
     syntax = A + (B + C)
     sql = "a b c"
-    ast = parse(syntax, sql, dialect='sqlite')
+    ast, bound = parse(syntax, sql, dialect='sqlite')
     print(ast)
-    generated = gen.generate(syntax, ast)
+    generated, bound = gen.generate(syntax, ast)
     assert ast == generated
     value, bmap = generated.bimap()
-    assert gen.generate(syntax, bmap(value)) == generated
+    u, v = gen.generate(syntax, bmap(value))
+    assert u == generated
 
 
 
@@ -106,24 +112,26 @@ def test_named_in_then():
     C = literal("c").mark("third")
     syntax = A + B + C
     sql = "a b c"
-    ast = parse(syntax, sql, dialect='sqlite')
+    ast, bound = parse(syntax, sql, dialect='sqlite')
     print(ast)
-    generated = gen.generate(syntax, ast)
+    generated, bound = gen.generate(syntax, ast)
     assert ast == generated
     value, bmap = generated.bimap()
-    assert gen.generate(syntax, bmap(value)) == generated
+    u, v = gen.generate(syntax, bmap(value))
+    assert u == generated
 
 
 def test_named_in_many():
     A = literal("x").mark("x")
     syntax = A.many()
     sql = "x x x"
-    ast = parse(syntax, sql, dialect='sqlite')
+    ast, bound = parse(syntax, sql, dialect='sqlite')
     print(ast)
-    generated = gen.generate(syntax, ast)
+    generated, bound = gen.generate(syntax, ast)
     assert ast == generated
     value, bmap = generated.bimap()
-    assert gen.generate(syntax, bmap(value)) == generated
+    u, v = gen.generate(syntax, bmap(value))
+    assert u == generated
 
 
 def test_named_in_or():
@@ -131,12 +139,13 @@ def test_named_in_or():
     B = literal("b").mark("b")
     syntax = A | B
     sql = "b"
-    ast = parse(syntax, sql, dialect='sqlite')
+    ast, bound = parse(syntax, sql, dialect='sqlite')
     print(ast)
-    generated = gen.generate(syntax, ast)
+    generated, bound = gen.generate(syntax, ast)
     assert ast == generated
     value, bmap = generated.bimap()
-    assert gen.generate(syntax, bmap(value)) == generated
+    u, v = gen.generate(syntax, bmap(value))
+    assert u == generated
 
 
 
@@ -148,21 +157,22 @@ def test_deep_mix():
     C = literal("c").mark("c")
     syntax = ((A + B) | C).many() + B
     sql = "a b a b c b"
-    ast = parse(syntax, sql, dialect='sqlite')
+    ast, bound = parse(syntax, sql, dialect='sqlite')
     print(ast)
-    generated = gen.generate(syntax, ast)
+    generated, bound = gen.generate(syntax, ast)
     print('---' * 40)
     print(generated)
     assert ast == generated
     value, bmap = generated.bimap()
-    assert gen.generate(syntax, bmap(value)) == generated
+    u, v = gen.generate(syntax, bmap(value))
+    assert u == generated
 
 
 def test_empty_many() -> None:
     A = literal("a")
     syntax = A.many()  # This should allow empty matches
     sql = ""
-    ast = parse(syntax, sql, dialect="sqlite")
+    ast, bound = parse(syntax, sql, dialect="sqlite")
     assert isinstance(ast, Error)
 
 
@@ -171,9 +181,10 @@ def test_backtracking_many() -> None:
     B = literal("b")
     syntax = (A.many() + B)  # must not eat the final "a" needed for B
     sql = "a a a a b"
-    ast = parse(syntax, sql, dialect="sqlite")
+    ast, bound = parse(syntax, sql, dialect="sqlite")
     value, bmap = ast.bimap()
-    assert gen.generate(syntax, bmap(value)) == ast
+    u, v = gen.generate(syntax, bmap(value))
+    assert u == ast
 
 def test_deep_nesting() -> None:
     A = literal("a")
@@ -181,7 +192,7 @@ def test_deep_nesting() -> None:
     for _ in range(100):
         syntax = syntax + A
     sql = " " .join("a" for _ in range(101))
-    ast = parse(syntax, sql, dialect="sqlite")
+    ast, bound = parse(syntax, sql, dialect="sqlite")
     assert ast is not None
 
 
@@ -189,7 +200,7 @@ def test_nested_many() -> None:
     A = literal("a")
     syntax = (A.many().many())  # groups of groups of "a"
     sql = "a a a"
-    ast = parse(syntax, sql, dialect="sqlite")
+    ast, bound = parse(syntax, sql, dialect="sqlite")
     assert isinstance(ast, Many)
 
 
@@ -197,9 +208,10 @@ def test_named_many() -> None:
     A = literal("a").mark("alpha")
     syntax = A.many()
     sql = "a a"
-    ast = parse(syntax, sql, dialect="sqlite")
+    ast, bound = parse(syntax, sql, dialect="sqlite")
     value, bmap = ast.bimap()
-    assert gen.generate(syntax, bmap(value)) == ast
+    u, v = gen.generate(syntax, bmap(value))
+    assert u == ast
 
 
 def test_or_named() -> None:
@@ -207,9 +219,10 @@ def test_or_named() -> None:
     B = literal("b").mark("y")
     syntax = A | B
     sql = "b"
-    ast = parse(syntax, sql, dialect="sqlite")
+    ast, bound = parse(syntax, sql, dialect="sqlite")
     value, bmap = ast.bimap()
-    assert gen.generate(syntax, bmap(value)) == ast
+    u, v = gen.generate(syntax, bmap(value))
+    assert u == ast
 
 
 def test_then_associativity() -> None:
@@ -218,7 +231,7 @@ def test_then_associativity() -> None:
     C = literal("c")
     syntax = A + B + C
     sql = "a b c"
-    ast = parse(syntax, sql, dialect="sqlite")
+    ast, bound = parse(syntax, sql, dialect="sqlite")
     # Should be Then(Then(A,B),C)
     assert ast == Then(kind=ThenKind.BOTH, 
                                    left=Then(kind=ThenKind.BOTH, 
@@ -232,7 +245,7 @@ def test_ambiguous() -> None:
     B = literal("a") + literal("b")
     syntax = A | B
     sql = "a"
-    ast = parse(syntax, sql, dialect="sqlite")
+    ast, bound = parse(syntax, sql, dialect="sqlite")
     # Does it prefer A (shorter) or B (fails)? Depends on design.
     assert ast == Choice[Token, Token](value=TokenGen.from_string("a"), kind=ChoiceKind.LEFT)
 
@@ -245,19 +258,19 @@ def test_combo() -> None:
     sql = "a b a b c b"
     # Should fail, as we discussed earlier
     # the working syntax should be ((A + B) | C).many() + B
-    ast = parse(syntax, sql, dialect="sqlite")
+    ast, bound = parse(syntax, sql, dialect="sqlite")
     assert isinstance(ast, Error)
-    ast = parse(((A + B) | C).many() + B, sql, dialect="sqlite")
+    ast, bound = parse(((A + B) | C).many() + B, sql, dialect="sqlite")
     assert not isinstance(ast, Error)
 
 
 def test_optional():
     A = literal("a").mark("a")
     syntax = A.optional()
-    ast1 = parse(syntax, "", dialect="sqlite")
+    ast1, bound = parse(syntax, "", dialect="sqlite")
     v1, _ = ast1.bimap()
     assert isinstance(v1, Nothing)
-    ast2 = parse(syntax, "a", dialect="sqlite")
+    ast2, bound = parse(syntax, "a", dialect="sqlite")
     v2, _ = ast2.bimap()
     assert v2 == Marked(name='a', value=TokenGen.from_string('a'))
 
