@@ -13,21 +13,23 @@ var = variable()
 def test_between()->None:
     sql = "then if then"
     syntax = IF.between(THEN, THEN)
-    ast:AST = parse(syntax, sql, dialect='sqlite')    
-    generated = gen.generate(syntax, ast)
+    ast, bound = parse(syntax, sql, dialect='sqlite')    
+    generated, bound = gen.generate(syntax, ast)
     assert ast == generated, "Parsed and generated results do not match."
     x, f = generated.bimap()
-    assert gen.generate(syntax, f(x)) == ast
+    u, v = gen.generate(syntax, f(x))
+    assert u == ast
 
 
 def test_sep_by()->None:
     sql = "if then if then if then if"
     syntax = IF.sep_by(THEN)
-    ast:AST = parse(syntax, sql, dialect='sqlite')    
-    generated = gen.generate(syntax, ast)
+    ast, bound = parse(syntax, sql, dialect='sqlite')    
+    generated, bound = gen.generate(syntax, ast)
     assert ast == generated, "Parsed and generated results do not match."
     x, f = generated.bimap()
-    assert gen.generate(syntax, f(x)) == ast
+    u, v = gen.generate(syntax, f(x))
+    assert u == ast
 
 def test_many_or()->None:
     IF = literal("if")
@@ -35,8 +37,9 @@ def test_many_or()->None:
     END = literal("end")
     syntax = (IF.many() | THEN.many()).many() // END
     sql = "if if then end"
-    ast:AST = parse(syntax, sql, dialect='sqlite')
-    generated = gen.generate(syntax, ast)
+    ast, bound = parse(syntax, sql, dialect='sqlite')
+    generated, bound = gen.generate(syntax, ast)
     assert ast == generated, "Parsed and generated results do not match."
     x, f = generated.bimap()
-    assert gen.generate(syntax, f(x)) == ast
+    u, v = gen.generate(syntax, f(x))
+    assert u == ast
